#!/usr/bin/perl
#
# BEGIN COPYRIGHT BLOCK
# This Program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; version 2 of the License.
# 
# This Program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along with
# this Program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place, Suite 330, Boston, MA 02111-1307 USA.
# 
# In addition, as a special exception, Red Hat, Inc. gives You the additional
# right to link the code of this Program with code not covered under the GNU
# General Public License ("Non-GPL Code") and to distribute linked combinations
# including the two, subject to the limitations in this paragraph. Non-GPL Code
# permitted under this exception must only link to the code of this Program
# through those well defined interfaces identified in the file named EXCEPTION
# found in the source code files (the "Approved Interfaces"). The files of
# Non-GPL Code may instantiate templates or use macros or inline functions from
# the Approved Interfaces without causing the resulting work to be covered by
# the GNU General Public License. Only Red Hat, Inc. may make changes or
# additions to the list of Approved Interfaces. You must obey the GNU General
# Public License in all respects for all of the Program code and other code used
# in conjunction with the Program except the Non-GPL Code covered by this
# exception. If you modify this file, you may extend this exception to your
# version of the file, but you are not obligated to do so. If you do not wish to
# provide this exception without modification, you must delete this exception
# statement from your version and license this file solely under the GPL without
# exception. 
# 
# 
# Copyright (C) 2001 Sun Microsystems, Inc. Used by permission.
# Copyright (C) 2005 Red Hat, Inc.
# All rights reserved.
# END COPYRIGHT BLOCK
#

use Getopt::Std;

$opt_D = "{{ROOT-DN}}";
$opt_p = "{{SERVER-PORT}}";
$opt_h = "{{SERVER-NAME}}";
$opt_v = 0;

sub usage {
    print(STDERR "Usage: $0 [-v] -D rootdn { -w password | -w - | -j filename } \n");
    print(STDERR "        -n instance [-t attributeName[:indextypes[:matchingrules]]]\n");
    print(STDERR " Opts: -D rootdn           - Directory Manager\n");
    print(STDERR "     : -w password         - Directory Manager's password\n");
    print(STDERR "     : -w -                - Prompt for Directory Manager's password\n");
    print(STDERR "     : -j filename         - Read Directory Manager's password from file\n");
    print(STDERR "     : -n instance         - instance to be indexed\n");
    print(STDERR "     : -t attributeName[:indextypes[:matchingrules]]\n");
    print(STDERR "                           - attributeName: name of the attribute to be indexed\n");
    print(STDERR "                             If omitted, all the indexes defined \n");
    print(STDERR "                             for that instance are generated.\n");
    print(STDERR "                           - indextypes: comma separated index types\n");
    print(STDERR "                           - matchingrules: comma separated matrules\n");
    print(STDERR "                           Example: -t foo:eq,pres\n");
    print(STDERR "     : -T vlvAttributeName - vlvAttributeName: name of the vlv attribute to be indexed\n");
    print(STDERR "     : -v                  - verbose\n");

    exit 1;
}

$instance = "";
$rootdn = "";
$passwd = "";
$passwdfile = "";
$attribute_arg = "";
$vlvattribute_arg = "";
$verbose = 0;

$prefix = "{{DS-ROOT}}";

$ENV{'PATH'} = "$prefix:$prefix/usr/bin::/usr/bin";

libpath_add("$prefix/usr/lib");
libpath_add("$prefix/usr/lib");
libpath_add("/usr/lib");
libpath_add("/usr/lib");

$ENV{'SHLIB_PATH'} = "$ENV{'LD_LIBRARY_PATH'}";

usage() if (!getopts('n:D:w:T:t:j:v'));

if ($opt_j ne ""){
    die "Error, cannot open password file $opt_j\n" unless (open (RPASS, $opt_j));
    $opt_w = <RPASS>;
    chomp($opt_w);
    close(RPASS);
}

usage() if( $opt_w eq "" );
usage() if( $#ARGV ge 0 );

$instance = $opt_n;
$rootdn = $opt_D;
$passwd = $opt_w;
$attribute_arg = $opt_t;
$vlvattribute_arg = $opt_T;
$verbose = $opt_v;

if ($passwdfile ne ""){
# Open file and get the password
    unless (open (RPASS, $passwdfile)) {
        die "Error, cannot open password file $passwdfile\n";
    }
    $passwd = <RPASS>;
    chomp($passwd);
    close(RPASS);
} elsif ($passwd eq "-"){
# Read the password from terminal
    print "Bind Password: ";
    # Disable console echo
    system("/bin/stty -echo") if -t STDIN;
    # read the answer
    $passwd = <STDIN>;
    # Enable console echo
    system("/bin/stty echo") if -t STDIN;
    print "\n";
    chop($passwd); # trim trailing newline
}

if ( $rootdn eq "" || $passwd eq "" ) 
{ 
    &usage; 
}

$vstr = "";
if ($verbose != 0) 
{ 
    $vstr = "-v"; 
}

($s, $m, $h, $dy, $mn, $yr, $wdy, $ydy, $r) = localtime(time);
$mn++; $yr += 1900;
$taskname = "db2index_${yr}_${mn}_${dy}_${h}_${m}_${s}";

if ( $instance eq "" )
{
    &usage;
}

# No attribute name has been specified: let's get them from the configuration
$attribute="";
$indexes_list="";
$vlvattribute="";
$vlvindexes_list="";
chdir("$prefix{{SEP}}usr{{SEP}}bin");
if ( $attribute_arg eq "" && $vlvattribute_arg eq "" )
{
    # Get the list of indexes from the entry
    $indexes_list="ldapsearch -x $vstr -h {{SERVER-NAME}} -p {{SERVER-PORT}} -D \"$rootdn\" -w \"$passwd\" -s one " .
    "-b \"cn=index,cn=\"$instance\", cn=ldbm database,cn=plugins,cn=config\" \"(&(objectclass=*)(nsSystemIndex=false))\" cn";
    # build the values of the attribute nsIndexAttribute
    open(LDAP1, "$indexes_list |");
    while (<LDAP1>) {
        s/\n //g;
        if (/^cn: (.*)\n/) {
            $IndexAttribute="nsIndexAttribute";
            $attribute="$attribute$IndexAttribute: $1\n";
        }
    }
    close(LDAP1);
    if ( $attribute eq "" )
    {
        # No attribute to index, just exit
        exit(0);
    }

    # Get the list of indexes from the entry
    $vlvindexes_list="ldapsearch -x $vstr -h {{SERVER-NAME}} -p {{SERVER-PORT}} -D \"$rootdn\" -w \"$passwd\" -s sub -b \"cn=\"$instance\", cn=ldbm database,cn=plugins,cn=config\" \"objectclass=vlvIndex\" cn";

    # build the values of the attribute nsIndexVlvAttribute
    open(LDAP1, "$vlvindexes_list |");
    while (<LDAP1>) {
        s/\n //g;
        if (/^cn: (.*)\n/) {
            $vlvIndexAttribute="nsIndexVlvAttribute";
            $vlvattribute="$vlvattribute$vlvIndexAttribute: $1\n";
        }
    }
    close(LDAP1);
}
else
{
    if ( $attribute_arg ne "" )
    {
        $attribute="nsIndexAttribute: $attribute_arg\n";
    }
    if ( $vlvattribute_arg ne "" )
    {
        $vlvattribute="nsIndexVlvAttribute: $vlvattribute_arg\n";
    }
}

# Build the task entry to add

$dn = "dn: cn=$taskname, cn=index, cn=tasks, cn=config\n";
$misc = "changetype: add\nobjectclass: top\nobjectclass: extensibleObject\n";
$cn =  "cn: $taskname\n";
$nsinstance = "nsInstance: ${instance}\n";

$entry = "${dn}${misc}${cn}${nsinstance}${attribute}${vlvattribute}";
open(FOO, "| ldapmodify -x $vstr -h {{SERVER-NAME}} -p {{SERVER-PORT}} -D \"$rootdn\" -w \"$passwd\" -a" );
print(FOO "$entry");
close(FOO);

sub libpath_add {
    my $libpath = shift;

    if ($libpath) {
        if ($ENV{'LD_LIBRARY_PATH'}) {
            $ENV{'LD_LIBRARY_PATH'} = "$ENV{'LD_LIBRARY_PATH'}:$libpath";
        } else {
            $ENV{'LD_LIBRARY_PATH'} = "$libpath";
        }
    }
}
